@@ -26,32 +26,55 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <stdbool.h>
+#include <errno.h>
 
 #include <glib.h>
 
+#include <ofono/handsfree-audio.h>
+
 #include "ofono.h"
 #include "common.h"
+#include "hfp.h"
 #include "gatserver.h"
 #include "gatppp.h"
 
 #define RING_TIMEOUT 3
 
+#define CVSD_OFFSET 0
+#define MSBC_OFFSET 1
+#define CODECS_COUNT (MSBC_OFFSET + 1)
+
+struct hfp_codec_info {
+	unsigned char type;
+	ofono_bool_t supported;
+};
+
 struct ofono_emulator {
 	struct ofono_atom *atom;
 	enum ofono_emulator_type type;
 	GAtServer *server;
 	GAtPPP *ppp;
-	gboolean slc;
 	int l_features;
 	int r_features;
-	int events_mode;
-	gboolean events_ind;
-	unsigned char cmee_mode;
 	GSList *indicators;
 	guint callsetup_source;
-	gboolean clip;
-	gboolean ccwa;
 	int pns_id;
+	struct ofono_handsfree_card *card;
+	struct hfp_codec_info r_codecs[CODECS_COUNT];
+	unsigned char selected_codec;
+	unsigned char negotiated_codec;
+	unsigned char proposed_codec;
+	ofono_emulator_codec_negotiation_cb codec_negotiation_cb;
+	void *codec_negotiation_data;
+	ofono_bool_t bac_received;
+	bool slc : 1;
+	unsigned int events_mode : 2;
+	bool events_ind : 1;
+	unsigned int cmee_mode : 2;
+	bool clip : 1;
+	bool ccwa : 1;
+	bool ddr_active : 1;
 };
 
 struct indicator {
@@ -100,9 +123,6 @@ static void cleanup_ppp(struct ofono_emulator *em)
 	__ofono_private_network_release(em->pns_id);
 	em->pns_id = 0;
 
-	if (em->server == NULL)
-		return;
-
 	g_at_server_resume(em->server);
 	g_at_server_send_final(em->server, G_AT_SERVER_RESULT_NO_CARRIER);
 }
@@ -455,7 +475,7 @@ static void brsf_cb(GAtServer *server, GAtServerRequestType type,
 		if (g_at_result_iter_next_number(&iter, &val) == FALSE)
 			goto fail;
 
-		if (val < 0 || val > 127)
+		if (val < 0 || val > 0xffff)
 			goto fail;
 
 		em->r_features = val;
@@ -638,7 +658,8 @@ done:
 
 		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
 
-		em->slc = TRUE;
+		__ofono_emulator_slc_condition(em,
+					OFONO_EMULATOR_SLC_CONDITION_CMER);
 		break;
 	}
 
@@ -828,6 +849,283 @@ fail:
 	}
 }
 
+static void bind_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *result, gpointer user_data)
+{
+	struct ofono_emulator *em = user_data;
+	char buf[128];
+
+	switch (type) {
+	case G_AT_SERVER_REQUEST_TYPE_QUERY:
+		g_at_server_send_info(em->server, "+BIND: 1,1", TRUE);
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
+
+		__ofono_emulator_slc_condition(em,
+					OFONO_EMULATOR_SLC_CONDITION_BIND);
+		break;
+
+	case G_AT_SERVER_REQUEST_TYPE_SUPPORT:
+		sprintf(buf, "+BIND: (1)");
+		g_at_server_send_info(em->server, buf, TRUE);
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
+		break;
+
+	case G_AT_SERVER_REQUEST_TYPE_SET:
+	{
+		GAtResultIter iter;
+		int hf_indicator;
+		int num_hf_indicators = 0;
+
+		g_at_result_iter_init(&iter, result);
+		g_at_result_iter_next(&iter, "");
+
+		/* check validity of the request */
+		while (num_hf_indicators < 20 &&
+				g_at_result_iter_next_number(&iter,
+							&hf_indicator)) {
+			if (hf_indicator > 0xffff)
+				goto fail;
+
+			num_hf_indicators += 1;
+		}
+
+		/* Check that we have nothing extra in the stream */
+		if (g_at_result_iter_skip_next(&iter) == TRUE)
+			goto fail;
+
+		/* request is valid, update the indicator activation status */
+		g_at_result_iter_init(&iter, result);
+		g_at_result_iter_next(&iter, "");
+
+		while (g_at_result_iter_next_number(&iter, &hf_indicator))
+			ofono_info("HF supports indicator: 0x%04x",
+					hf_indicator);
+
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
+
+		break;
+	}
+
+	default:
+fail:
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
+		break;
+	}
+}
+
+static void biev_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *result, gpointer user_data)
+{
+	struct ofono_emulator *em = user_data;
+
+	switch (type) {
+	case G_AT_SERVER_REQUEST_TYPE_SET:
+	{
+		GAtResultIter iter;
+		int hf_indicator;
+		int val;
+
+		g_at_result_iter_init(&iter, result);
+		g_at_result_iter_next(&iter, "");
+
+		if (g_at_result_iter_next_number(&iter, &hf_indicator) == FALSE)
+			goto fail;
+
+		if (hf_indicator != HFP_HF_INDICATOR_ENHANCED_SAFETY)
+			goto fail;
+
+		if (em->ddr_active == FALSE)
+			goto fail;
+
+		if (g_at_result_iter_next_number(&iter, &val) == FALSE)
+			goto fail;
+
+		if (val < 0 || val > 1)
+			goto fail;
+
+		ofono_info("Enhanced Safety indicator: %d", val);
+
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
+		break;
+	}
+
+	default:
+fail:
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
+		break;
+	}
+}
+
+static void finish_codec_negotiation(struct ofono_emulator *em,
+			int err)
+{
+	if (em->codec_negotiation_cb == NULL)
+		return;
+
+	em->codec_negotiation_cb(err, em->codec_negotiation_data);
+
+	em->codec_negotiation_cb = NULL;
+	em->codec_negotiation_data = NULL;
+}
+
+static void bac_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *result, gpointer user_data)
+{
+	struct ofono_emulator *em = user_data;
+	GAtResultIter iter;
+	int val;
+
+	DBG("");
+
+	switch (type) {
+	case G_AT_SERVER_REQUEST_TYPE_SET:
+		g_at_result_iter_init(&iter, result);
+		g_at_result_iter_next(&iter, "");
+
+		/*
+		 * CVSD codec is mandatory and must come first.
+		 * See HFP v1.6 4.34.1
+		 */
+		if (g_at_result_iter_next_number(&iter, &val) == FALSE ||
+				val != HFP_CODEC_CVSD)
+			goto fail;
+
+		em->bac_received = TRUE;
+
+		em->negotiated_codec = 0;
+		em->r_codecs[CVSD_OFFSET].supported = TRUE;
+
+		while (g_at_result_iter_next_number(&iter, &val)) {
+			switch (val) {
+			case HFP_CODEC_MSBC:
+				em->r_codecs[MSBC_OFFSET].supported = TRUE;
+				break;
+			default:
+				DBG("Unsupported HFP codec %d", val);
+				break;
+			}
+		}
+
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
+
+		/*
+		 * If we're currently in the process of selecting a codec
+		 * we have to restart that now
+		 */
+		if (em->proposed_codec) {
+			em->proposed_codec = 0;
+			ofono_emulator_start_codec_negotiation(em, NULL, NULL);
+		}
+
+		break;
+
+	default:
+fail:
+		DBG("Process AT+BAC failed");
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
+
+		finish_codec_negotiation(em, -EIO);
+
+		break;
+	}
+}
+
+static void connect_sco(struct ofono_emulator *em)
+{
+	int err;
+
+	DBG("");
+
+	if (em->card == NULL) {
+		finish_codec_negotiation(em, -EINVAL);
+		return;
+	}
+
+	err = ofono_handsfree_card_connect_sco(em->card);
+	if (err == 0) {
+		finish_codec_negotiation(em, 0);
+		return;
+	}
+
+	/* If we have another codec we can try then lets do that */
+	if (em->negotiated_codec != HFP_CODEC_CVSD) {
+		em->selected_codec = HFP_CODEC_CVSD;
+		ofono_emulator_start_codec_negotiation(em,
+					em->codec_negotiation_cb,
+					em->codec_negotiation_data);
+		return;
+	}
+
+	finish_codec_negotiation(em, -EIO);
+}
+
+static void bcs_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *result, gpointer user_data)
+{
+	struct ofono_emulator *em = user_data;
+	GAtResultIter iter;
+	int val;
+
+	switch (type) {
+	case G_AT_SERVER_REQUEST_TYPE_SET:
+		g_at_result_iter_init(&iter, result);
+		g_at_result_iter_next(&iter, "");
+
+		if (!g_at_result_iter_next_number(&iter, &val))
+			break;
+
+		if (em->proposed_codec != val) {
+			em->proposed_codec = 0;
+			break;
+		}
+
+		em->proposed_codec = 0;
+		em->negotiated_codec = val;
+
+		DBG("negotiated codec %d", val);
+
+		if (em->card != NULL)
+			ofono_handsfree_card_set_codec(em->card,
+							em->negotiated_codec);
+
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
+
+		connect_sco(em);
+
+		return;
+	default:
+		break;
+	}
+
+	finish_codec_negotiation(em, -EIO);
+
+	g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
+}
+
+static void bcc_cb(GAtServer *server, GAtServerRequestType type,
+			GAtResult *result, gpointer user_data)
+{
+	struct ofono_emulator *em = user_data;
+
+	switch (type) {
+	case G_AT_SERVER_REQUEST_TYPE_COMMAND_ONLY:
+		g_at_server_send_final(server, G_AT_SERVER_RESULT_OK);
+
+		if (!em->negotiated_codec) {
+			ofono_emulator_start_codec_negotiation(em, NULL, NULL);
+			return;
+		}
+
+		connect_sco(em);
+
+		return;
+	default:
+		break;
+	}
+
+	g_at_server_send_final(server, G_AT_SERVER_RESULT_ERROR);
+}
+
 static void emulator_add_indicator(struct ofono_emulator *em, const char* name,
 					int min, int max, int dflt,
 					gboolean mandatory)
@@ -882,6 +1180,9 @@ static void emulator_unregister(struct ofono_atom *atom)
 
 	g_at_server_unref(em->server);
 	em->server = NULL;
+
+	ofono_handsfree_card_remove(em->card);
+	em->card = NULL;
 }
 
 void ofono_emulator_register(struct ofono_emulator *em, int fd)
@@ -908,6 +1209,8 @@ void ofono_emulator_register(struct ofono_emulator *em, int fd)
 						em);
 
 	if (em->type == OFONO_EMULATOR_TYPE_HFP) {
+		em->ddr_active = true;
+
 		emulator_add_indicator(em, OFONO_EMULATOR_IND_SERVICE, 0, 1, 0,
 									FALSE);
 		emulator_add_indicator(em, OFONO_EMULATOR_IND_CALL, 0, 1, 0,
@@ -930,6 +1233,11 @@ void ofono_emulator_register(struct ofono_emulator *em, int fd)
 		g_at_server_register(em->server, "+CCWA", ccwa_cb, em, NULL);
 		g_at_server_register(em->server, "+CMEE", cmee_cb, em, NULL);
 		g_at_server_register(em->server, "+BIA", bia_cb, em, NULL);
+		g_at_server_register(em->server, "+BIND", bind_cb, em, NULL);
+		g_at_server_register(em->server, "+BIEV", biev_cb, em, NULL);
+		g_at_server_register(em->server, "+BAC", bac_cb, em, NULL);
+		g_at_server_register(em->server, "+BCC", bcc_cb, em, NULL);
+		g_at_server_register(em->server, "+BCS", bcs_cb, em, NULL);
 	}
 
 	__ofono_atom_register(em->atom, emulator_unregister);
@@ -983,6 +1291,8 @@ struct ofono_emulator *ofono_emulator_create(struct ofono_modem *modem,
 	em->l_features |= HFP_AG_FEATURE_ENHANCED_CALL_STATUS;
 	em->l_features |= HFP_AG_FEATURE_ENHANCED_CALL_CONTROL;
 	em->l_features |= HFP_AG_FEATURE_EXTENDED_RES_CODE;
+	em->l_features |= HFP_AG_FEATURE_HF_INDICATORS;
+	em->l_features |= HFP_AG_FEATURE_CODEC_NEGOTIATION;
 	em->events_mode = 3;	/* default mode is forwarding events */
 	em->cmee_mode = 0;	/* CME ERROR disabled by default */
 
@@ -1037,6 +1347,7 @@ void ofono_emulator_send_final(struct ofono_emulator *em,
 	case OFONO_ERROR_TYPE_CEER:
 	case OFONO_ERROR_TYPE_SIM:
 	case OFONO_ERROR_TYPE_FAILURE:
+	case OFONO_ERROR_TYPE_ERRNO:
 failure:
 		g_at_server_send_final(em->server, G_AT_SERVER_RESULT_ERROR);
 		break;
@@ -1074,16 +1385,11 @@ struct ofono_emulator_request {
 };
 
 static void handler_proxy(GAtServer *server, GAtServerRequestType type,
-				GAtResult *result, gpointer userdata)
+					GAtResult *result, gpointer userdata)
 {
 	struct handler *h = userdata;
 	struct ofono_emulator_request req;
 
-	if (h->em->type == OFONO_EMULATOR_TYPE_HFP && h->em->slc == FALSE) {
-		g_at_server_send_final(h->em->server, G_AT_SERVER_RESULT_ERROR);
-		return;
-	}
-
 	switch (type) {
 	case G_AT_SERVER_REQUEST_TYPE_COMMAND_ONLY:
 		req.type = OFONO_EMULATOR_REQUEST_TYPE_COMMAND_ONLY;
@@ -1104,6 +1410,33 @@ static void handler_proxy(GAtServer *server, GAtServerRequestType type,
 	h->cb(h->em, &req, h->data);
 }
 
+static void handler_proxy_need_slc(GAtServer *server,
+					GAtServerRequestType type,
+					GAtResult *result, gpointer userdata)
+{
+	struct handler *h = userdata;
+
+	if (h->em->slc == FALSE) {
+		g_at_server_send_final(h->em->server, G_AT_SERVER_RESULT_ERROR);
+		return;
+	}
+
+	handler_proxy(server, type, result, userdata);
+}
+
+static void handler_proxy_chld(GAtServer *server, GAtServerRequestType type,
+				GAtResult *result, gpointer userdata)
+{
+	struct handler *h = userdata;
+
+	if (h->em->slc == FALSE && type != G_AT_SERVER_REQUEST_TYPE_SUPPORT) {
+		g_at_server_send_final(h->em->server, G_AT_SERVER_RESULT_ERROR);
+		return;
+	}
+
+	handler_proxy(server, type, result, userdata);
+}
+
 static void handler_destroy(gpointer userdata)
 {
 	struct handler *h = userdata;
@@ -1120,6 +1453,7 @@ ofono_bool_t ofono_emulator_add_handler(struct ofono_emulator *em,
 					void *data, ofono_destroy_func destroy)
 {
 	struct handler *h;
+	GAtServerNotifyFunc func = handler_proxy;
 
 	h = g_new0(struct handler, 1);
 	h->cb = cb;
@@ -1127,7 +1461,14 @@ ofono_bool_t ofono_emulator_add_handler(struct ofono_emulator *em,
 	h->destroy = destroy;
 	h->em = em;
 
-	if (g_at_server_register(em->server, prefix, handler_proxy, h,
+	if (em->type == OFONO_EMULATOR_TYPE_HFP) {
+		func = handler_proxy_need_slc;
+
+		if (!strcmp(prefix, "+CHLD"))
+			func = handler_proxy_chld;
+	}
+
+	if (g_at_server_register(em->server, prefix, func, h,
 					handler_destroy) == TRUE)
 		return TRUE;
 
@@ -1268,3 +1609,131 @@ void __ofono_emulator_set_indicator_forced(struct ofono_emulator *em,
 			ind->deferred = TRUE;
 	}
 }
+
+void __ofono_emulator_slc_condition(struct ofono_emulator *em,
+					enum ofono_emulator_slc_condition cond)
+{
+	if (em->slc == TRUE)
+		return;
+
+	switch (cond) {
+	case OFONO_EMULATOR_SLC_CONDITION_CMER:
+		if ((em->r_features & HFP_HF_FEATURE_3WAY) &&
+				(em->l_features & HFP_AG_FEATURE_3WAY))
+			return;
+		/* Fall Through */
+
+	case OFONO_EMULATOR_SLC_CONDITION_CHLD:
+		if ((em->r_features & HFP_HF_FEATURE_HF_INDICATORS) &&
+				(em->l_features & HFP_HF_FEATURE_HF_INDICATORS))
+			return;
+		/* Fall Through */
+
+	case OFONO_EMULATOR_SLC_CONDITION_BIND:
+		ofono_info("SLC reached");
+		em->slc = TRUE;
+
+		ofono_handsfree_card_register(em->card);
+
+	default:
+		break;
+	}
+}
+
+void ofono_emulator_set_hf_indicator_active(struct ofono_emulator *em,
+						int indicator,
+						ofono_bool_t active)
+{
+	char buf[64];
+
+	if (!(em->l_features & HFP_HF_FEATURE_HF_INDICATORS))
+		return;
+
+	if (!(em->r_features & HFP_HF_FEATURE_HF_INDICATORS))
+		return;
+
+	if (indicator != HFP_HF_INDICATOR_ENHANCED_SAFETY)
+		return;
+
+	em->ddr_active = active;
+
+	sprintf(buf, "+BIND: %d,%d", HFP_HF_INDICATOR_ENHANCED_SAFETY, active);
+	g_at_server_send_unsolicited(em->server, buf);
+}
+
+void ofono_emulator_set_handsfree_card(struct ofono_emulator *em,
+					struct ofono_handsfree_card *card)
+{
+	if (em == NULL)
+		return;
+
+	em->card = card;
+}
+
+static unsigned char select_codec(struct ofono_emulator *em)
+{
+	if (ofono_handsfree_audio_has_wideband() &&
+			em->r_codecs[MSBC_OFFSET].supported)
+		return HFP_CODEC_MSBC;
+
+	/* CVSD is mandatory for both sides */
+	return HFP_CODEC_CVSD;
+}
+
+int ofono_emulator_start_codec_negotiation(struct ofono_emulator *em,
+			ofono_emulator_codec_negotiation_cb cb, void *data)
+{
+	char buf[64];
+	unsigned char codec;
+
+	if (em == NULL)
+		return -EINVAL;
+
+	if (cb != NULL && em->codec_negotiation_cb != NULL)
+		return -EALREADY;
+
+	if (em->proposed_codec > 0)
+		return -EALREADY;
+
+	if (!em->bac_received || em->negotiated_codec > 0) {
+		/*
+		 * Report we're done even if we don't have done any
+		 * negotiation as the other side may have to clean up.
+		 */
+		cb(0, data);
+
+		/*
+		 * If we didn't received any +BAC during the SLC setup the
+		 * remote side doesn't support codec negotiation and we can
+		 * directly connect our card. Otherwise if we got +BAC and
+		 * already have a negotiated codec we can proceed here
+		 * without doing any negotiation again.
+		 */
+		ofono_handsfree_card_connect_sco(em->card);
+
+		return 0;
+	}
+
+	if (em->selected_codec > 0) {
+		codec = em->selected_codec;
+		em->selected_codec = 0;
+		goto done;
+	}
+
+	codec = select_codec(em);
+	if (!codec) {
+		DBG("Failed to select HFP codec");
+		return -EINVAL;
+	}
+
+done:
+	em->proposed_codec = codec;
+
+	em->codec_negotiation_cb = cb;
+	em->codec_negotiation_data = data;
+
+	snprintf(buf, 64, "+BCS: %d", em->proposed_codec);
+	g_at_server_send_unsolicited(em->server, buf);
+
+	return 0;
+}
