@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
@@ -35,7 +34,6 @@
 
 #include "common.h"
 #include "smsutil.h"
-#include "util.h"
 
 #define MAX_USSD_LENGTH 160
 
